@@ -29,12 +29,20 @@
   incomplete_features
 )]
 #![feature(
+  const_evaluatable_checked,
   const_fn,
+  const_fn_floating_point_arithmetic,
+  const_fn_fn_ptr_basics,
   const_fn_union,
   const_generics,
+  const_intrinsic_copy,
   const_mut_refs,
   const_panic,
+  const_ptr_is_null,
+  const_ptr_offset,
   const_ptr_offset_from,
+  const_ptr_read,
+  const_ptr_write,
   const_raw_ptr_deref,
   const_raw_ptr_to_usize_cast,
   const_slice_from_raw_parts,
@@ -42,6 +50,7 @@
   core_intrinsics,
   doc_cfg,
   exact_size_is_empty,
+  inline_const,
   maybe_uninit_extra,
   maybe_uninit_ref,
   maybe_uninit_uninit_array,
@@ -100,8 +109,11 @@ pub mod utils;
 /// A [`Vec`](alloc::vec::Vec)-like struct (mostly directly API-compatible where it can be)
 /// implemented with const generics around an array of fixed `N` capacity.
 pub struct StaticVec<T, const N: usize> {
-  // We create this field in an uninitialized state, and write to it element-wise as needed
-  // via pointer methods. At no time should `assume_init` *ever* be called through it.
+  // We create this field in an uninitialized state, and write to it element-wise as needed via
+  // pointer methods. At no time should the regular `assume_init` function *ever* be called through
+  // it unless an abundantly obviously-not-UB reason is present or a clear explanation is provided
+  // in comments. At this time, we only use `assume_init_read` and `assume_init_mut` (soundly, and
+  // for good reasons) in about two places total.
   data: MaybeUninit<[T; N]>,
   // The constant `N` parameter (and thus the total span of `data`) represent capacity for us,
   // while the field below represents, as its name suggests, the current length of a StaticVec
@@ -184,9 +196,9 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// let v2 = StaticVec::<i32, 3>::new_from_array([1, 2, 3, 4, 5, 6]);
   /// assert_eq!(v2, [1, 2, 3]);
   /// ```
-  /// Note that StaticVec also implements [`From`](core::convert::From) for both slices
-  /// and static arrays, which may prove more ergonomic in some cases as it allows
-  /// for a greater degree of type inference:
+  /// Note that StaticVec also implements [`From`](core::convert::From) for both slices and static
+  /// arrays (as well as several other types), which may prove more ergonomic in some cases as it
+  /// allows for a greater degree of type inference:
   /// ```
   /// # use staticvec::StaticVec;
   /// // The StaticVec on the next line is inferred to be of type `StaticVec<&'static str, 4>`.
@@ -360,15 +372,22 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(vec, data);
   /// ```
   #[inline(always)]
-  pub unsafe fn set_len(&mut self, new_len: usize) {
+  pub const unsafe fn set_len(&mut self, new_len: usize) {
     // Most of the `unsafe` functions in this crate that are heavily used internally
     // have debug-build-only assertions where it's useful.
+    /*
+    // The formatted assertion macros are not const-compatible yet.
     debug_assert!(
       new_len <= N,
       "In `StaticVec::set_len`, provided length {} exceeds the maximum capacity of {}!",
       new_len,
       N
     );
+    */
+    debug_assert!(
+      new_len <= N,
+      "A `new_len` greater than `N` was passed to `StaticVec::set_len`!"
+    );
     self.length = new_len;
   }
 
@@ -510,18 +529,27 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// }
   /// ```
   #[inline(always)]
-  pub unsafe fn ptr_at_unchecked(&self, index: usize) -> *const T {
-    // We check against `N` as opposed to `length` in our debug assertion here, as these
-    // `_unchecked` versions of `ptr_at` and `mut_ptr_at` are primarily intended for
-    // initialization-related purposes (and used extensively that way internally throughout the
-    // crate.)
+  pub const unsafe fn ptr_at_unchecked(&self, index: usize) -> *const T {
+    // We (inclusively, to account for the possibility of `N` being 0) check against `N` as opposed
+    // to `length` in our debug assertion here, as these `_unchecked` versions of `ptr_at` and
+    // `mut_ptr_at` are primarily intended for initialization-related purposes (and used extensively
+    // that way internally throughout the crate.)
+    /*
+    // The formatted assertion macros are not const-compatible yet.
     debug_assert!(
       index <= N,
       "In `StaticVec::ptr_at_unchecked`, provided index {} must be within `0..={}`!",
       index,
       N
     );
-    self.as_ptr().add(index)
+    */
+    debug_assert!(
+      index <= N,
+      "Bounds check failure in `StaticVec::ptr_at_unchecked`!",
+    );
+    // Safety: `index` is explicitly a `usize` to start with, and so cannot be negative at this
+    // point.
+    self.as_ptr().offset(index as isize)
   }
 
   /// Returns a mutable pointer to the element of the StaticVec at `index` without doing any
@@ -550,18 +578,27 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, ["I", "am", "a", "StaticVec!"]);
   /// ```
   #[inline(always)]
-  pub unsafe fn mut_ptr_at_unchecked(&mut self, index: usize) -> *mut T {
-    // We check against `N` as opposed to `length` in our debug assertion here, as these
-    // `_unchecked` versions of `ptr_at` and `mut_ptr_at` are primarily intended for
-    // initialization-related purposes (and used extensively that way internally throughout the
-    // crate.)
+  pub const unsafe fn mut_ptr_at_unchecked(&mut self, index: usize) -> *mut T {
+    // We (inclusively, to account for the possibility of `N` being 0) check against `N` as opposed
+    // to `length` in our debug assertion here, as these `_unchecked` versions of `ptr_at` and
+    // `mut_ptr_at` are primarily intended for initialization-related purposes (and used extensively
+    // that way internally throughout the crate.)
+    /*
+    // The formatted assertion macros are not const-compatible yet.
     debug_assert!(
       index <= N,
       "In `StaticVec::mut_ptr_at_unchecked`, provided index {} must be within `0..={}`!",
       index,
       N
     );
-    self.as_mut_ptr().add(index)
+    */
+    debug_assert!(
+      index <= N,
+      "Bounds check failure in `StaticVec::mut_ptr_at_unchecked`!",
+    );
+    // Safety: `index` is explicitly a `usize` to start with, and so cannot be negative at this
+    // point.
+    self.as_mut_ptr().offset(index as isize)
   }
 
   /// Returns a constant pointer to the element of the StaticVec at `index` if `index`
@@ -576,13 +613,20 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// unsafe { assert_eq!(*p, "StaticVec!") };
   /// ```
   #[inline(always)]
-  pub fn ptr_at(&self, index: usize) -> *const T {
+  pub const fn ptr_at(&self, index: usize) -> *const T {
+    /*
+    // The formatted assertion macros are not const-compatible yet.
     assert!(
       index < self.length,
       "In `StaticVec::ptr_at`, provided index {} must be within `0..{}`!",
       index,
       self.length
     );
+    */
+    assert!(
+      index < self.length,
+      "Bounds check failure in `StaticVec::ptr_at`!"
+    );
     unsafe { self.ptr_at_unchecked(index) }
   }
 
@@ -599,13 +643,20 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, ["I", "am", "a", "StaticVec!"]);
   /// ```
   #[inline(always)]
-  pub fn mut_ptr_at(&mut self, index: usize) -> *mut T {
+  pub const fn mut_ptr_at(&mut self, index: usize) -> *mut T {
+    /*
+    // The formatted assertion macros are not const-compatible yet.
     assert!(
       index < self.length,
       "In `StaticVec::mut_ptr_at`, provided index {} must be within `0..{}`!",
       index,
       self.length
     );
+    */
+    assert!(
+      index < self.length,
+      "Bounds check failure in `StaticVec::mut_ptr_at`!"
+    );
     unsafe { self.mut_ptr_at_unchecked(index) }
   }
 
@@ -627,15 +678,22 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// unsafe { assert_eq!(*staticvec![1, 2, 3].get_unchecked(1), 2) };
   /// ```
   #[inline(always)]
-  pub unsafe fn get_unchecked(&self, index: usize) -> &T {
+  pub const unsafe fn get_unchecked(&self, index: usize) -> &T {
     // This function is used internally in places where `length` has been intentionally
     // temporarily set to zero, so we do our debug assertion against `N`.
+    /*
+    // The formatted assertion macros are not const-compatible yet.
     debug_assert!(
       index < N,
       "In `StaticVec::get_unchecked`, provided index {} must be within `0..{}`!",
       index,
       N
     );
+    */
+    debug_assert!(
+      index < N,
+      "Bounds check failure in `StaticVec::get_unchecked`!"
+    );
     &*self.ptr_at_unchecked(index)
   }
 
@@ -658,15 +716,22 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, [1, 9, 3]);
   /// ```
   #[inline(always)]
-  pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T {
+  pub const unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T {
     // This function is used internally in places where `length` has been intentionally
     // temporarily set to zero, so we do our debug assertion against `N`.
+    /*
+    // The formatted assertion macros are not const-compatible yet.
     debug_assert!(
       index < N,
       "In `StaticVec::get_unchecked_mut`, provided index {} must be within `0..{}`!",
       index,
       N
     );
+    */
+    debug_assert!(
+      index < N,
+      "Bounds check failure in `StaticVec::get_unchecked_mut`!"
+    );
     &mut *self.mut_ptr_at_unchecked(index)
   }
 
@@ -687,7 +752,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, [1, 2, 3]);
   /// ```
   #[inline(always)]
-  pub unsafe fn push_unchecked(&mut self, value: T) {
+  pub const unsafe fn push_unchecked(&mut self, value: T) {
     debug_assert!(
       self.is_not_full(),
       "`StaticVec::push_unchecked` was called through a StaticVec already at maximum capacity!"
@@ -714,7 +779,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, [1, 2, 3]);
   /// ```
   #[inline(always)]
-  pub unsafe fn pop_unchecked(&mut self) -> T {
+  pub const unsafe fn pop_unchecked(&mut self) -> T {
     debug_assert!(
       self.is_not_empty(),
       "`StaticVec::pop_unchecked` was called through an empty StaticVec!"
@@ -736,7 +801,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert!(v2.try_push(1).is_ok());
   /// ```
   #[inline(always)]
-  pub fn try_push(&mut self, value: T) -> Result<(), PushCapacityError<T, N>> {
+  pub const fn try_push(&mut self, value: T) -> Result<(), PushCapacityError<T, N>> {
     if self.is_not_full() {
       unsafe { self.push_unchecked(value) };
       Ok(())
@@ -757,7 +822,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, [1, 2]);
   /// ```
   #[inline(always)]
-  pub fn push(&mut self, value: T) {
+  pub const fn push(&mut self, value: T) {
     assert!(
       self.is_not_full(),
       "`StaticVec::push` was called through a StaticVec already at maximum capacity!"
@@ -777,7 +842,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, [1, 2]);
   /// ```
   #[inline(always)]
-  pub fn pop(&mut self) -> Option<T> {
+  pub const fn pop(&mut self) -> Option<T> {
     if self.is_empty() {
       None
     } else {
@@ -797,7 +862,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(None, v2.first());
   /// ```
   #[inline(always)]
-  pub fn first(&self) -> Option<&T> {
+  pub const fn first(&self) -> Option<&T> {
     if self.is_empty() {
       None
     } else {
@@ -818,7 +883,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(x, &[5, 1, 2]);
   /// ```
   #[inline(always)]
-  pub fn first_mut(&mut self) -> Option<&mut T> {
+  pub const fn first_mut(&mut self) -> Option<&mut T> {
     if self.is_empty() {
       None
     } else {
@@ -838,7 +903,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(None, w.last());
   /// ```
   #[inline(always)]
-  pub fn last(&self) -> Option<&T> {
+  pub const fn last(&self) -> Option<&T> {
     if self.is_empty() {
       None
     } else {
@@ -859,7 +924,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(x, &[0, 1, 10]);
   /// ```
   #[inline(always)]
-  pub fn last_mut(&mut self) -> Option<&mut T> {
+  pub const fn last_mut(&mut self) -> Option<&mut T> {
     if self.is_empty() {
       None
     } else {
@@ -870,12 +935,12 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// A crate-local unchecked version of `remove`, currently only used in the implementation of
   /// `StaticVecSplice`.
   #[inline]
-  pub(crate) fn remove_unchecked(&mut self, index: usize) -> T {
+  pub(crate) const fn remove_unchecked(&mut self, index: usize) -> T {
     let old_length = self.length;
     unsafe {
       let self_ptr = self.mut_ptr_at_unchecked(index);
       let res = self_ptr.read();
-      self_ptr.offset(1).copy_to(self_ptr, old_length - index - 1);
+      ptr::copy(self_ptr.offset(1), self_ptr, old_length - index - 1);
       self.set_len(old_length - 1);
       res
     }
@@ -891,7 +956,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(staticvec![1, 2, 3].remove(1), 2);
   /// ```
   #[inline]
-  pub fn remove(&mut self, index: usize) -> T {
+  pub const fn remove(&mut self, index: usize) -> T {
     let old_length = self.length;
     assert!(
       index < old_length,
@@ -900,7 +965,7 @@ impl<T, const N: usize> StaticVec<T, N> {
     unsafe {
       let self_ptr = self.mut_ptr_at_unchecked(index);
       let res = self_ptr.read();
-      self_ptr.offset(1).copy_to(self_ptr, old_length - index - 1);
+      ptr::copy(self_ptr.offset(1), self_ptr, old_length - index - 1);
       self.set_len(old_length - 1);
       res
     }
@@ -986,7 +1051,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, [1, 4, 2, 3]);
   /// ```
   #[inline]
-  pub fn insert(&mut self, index: usize, value: T) {
+  pub const fn insert(&mut self, index: usize, value: T) {
     let old_length = self.length;
     assert!(
       old_length < N && index <= old_length,
@@ -994,7 +1059,7 @@ impl<T, const N: usize> StaticVec<T, N> {
     );
     unsafe {
       let self_ptr = self.mut_ptr_at_unchecked(index);
-      self_ptr.copy_to(self_ptr.offset(1), old_length - index);
+      ptr::copy(self_ptr, self_ptr.offset(1), old_length - index);
       self_ptr.write(value);
       self.set_len(old_length + 1);
     }
@@ -1072,7 +1137,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v, [1, 2, 3, 4, 5, 6, 7, 8]);
   /// ```
   #[inline]
-  pub fn insert_from_slice(&mut self, index: usize, values: &[T])
+  pub const fn insert_from_slice(&mut self, index: usize, values: &[T])
   where T: Copy {
     let old_length = self.length;
     let values_length = values.len();
@@ -1082,8 +1147,8 @@ impl<T, const N: usize> StaticVec<T, N> {
     );
     unsafe {
       let self_ptr = self.mut_ptr_at_unchecked(index);
-      self_ptr.copy_to(self_ptr.add(values_length), old_length - index);
-      self_ptr.copy_from_nonoverlapping(values.as_ptr(), values_length);
+      ptr::copy(self_ptr, self_ptr.add(values_length), old_length - index);
+      ptr::copy_nonoverlapping(values.as_ptr(), self_ptr, values_length);
       self.set_len(old_length + values_length);
     }
   }
@@ -1100,11 +1165,13 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// ```
   #[inline]
   pub fn try_insert(&mut self, index: usize, value: T) -> Result<(), CapacityError<N>> {
+    // This one seems like it *should* be `const fn` compatible, but for some reason rustc`
+    // gives a "can't evaluate destructors" error about it currently,
     let old_length = self.length;
     if old_length < N && index <= old_length {
       unsafe {
         let self_ptr = self.mut_ptr_at_unchecked(index);
-        self_ptr.copy_to(self_ptr.offset(1), old_length - index);
+        ptr::copy(self_ptr, self_ptr.offset(1), old_length - index);
         self_ptr.write(value);
         self.set_len(old_length + 1);
       }
@@ -1128,7 +1195,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert!(v2.try_insert_from_slice(207, &[5, 6]).is_err());
   /// ```
   #[inline]
-  pub fn try_insert_from_slice(
+  pub const fn try_insert_from_slice(
     &mut self,
     index: usize,
     values: &[T],
@@ -1141,8 +1208,8 @@ impl<T, const N: usize> StaticVec<T, N> {
     if old_length < N && index <= old_length && values_length <= self.remaining_capacity() {
       unsafe {
         let self_ptr = self.mut_ptr_at_unchecked(index);
-        self_ptr.copy_to(self_ptr.add(values_length), old_length - index);
-        self_ptr.copy_from_nonoverlapping(values.as_ptr(), values_length);
+        ptr::copy(self_ptr, self_ptr.add(values_length), old_length - index);
+        ptr::copy_nonoverlapping(values.as_ptr(), self_ptr, values_length);
         self.set_len(old_length + values_length);
       }
       Ok(())
@@ -1538,7 +1605,6 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
   }
 
-  /*
   /// Returns a new StaticVec consisting of the elements of `self` and `other` concatenated in
   /// linear fashion such that the first element of `other` comes immediately after the last
   /// element of `self`.
@@ -1607,9 +1673,7 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
     res
   }
-  */
 
-  /*
   /// Returns a new StaticVec consisting of the elements of `self` in linear order, interspersed
   /// with a copy of `separator` between each one.
   ///
@@ -1626,19 +1690,19 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// );
   /// ```
   #[inline]
-  pub fn intersperse(&self, separator: T) -> StaticVec<T, { N * 2 }>
+  pub const fn intersperse(&self, separator: T) -> StaticVec<T, { N * 2 }>
   where T: Copy {
     if self.is_empty() {
       return StaticVec::new();
     }
-    let mut res = StaticVec::new();
+    let mut res = StaticVec::new_data_uninit();
     // The `as *mut T` cast below is necessary to make the type inference work properly (at the
     // moment at least). `rustc` still gets a bit confused by math operations done on const generic
     // values in return types it seems.
     // Note that the `StaticVec::new()` calls above *have* to be written without any constraints,
     // as otherwise we'll hit a particular bug where `rustc` says:
     // "expected struct `StaticVec<_, { N * 2 }>`, found struct `StaticVec<_, { N * 2 }>`".
-    let mut res_ptr = res.as_mut_ptr() as *mut T;
+    let mut res_ptr = StaticVec::first_ptr_mut(&mut res) as *mut T;
     let mut i = 0;
     let length = self.length;
     while i < length - 1 {
@@ -1651,9 +1715,11 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
     unsafe {
       res_ptr.write(self.ptr_at_unchecked(i).read());
-      res.set_len((length * 2) - 1);
     }
-    res
+    StaticVec {
+      data: res,
+      length: (length * 2) - 1,
+    }
   }
 
   /// A version of [`intersperse`](crate::StaticVec::intersperse) for scenarios where `T` does not
@@ -1689,7 +1755,6 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
     res
   }
-  */
 
   /// Returns a StaticVec containing the contents of a [`Vec`](alloc::vec::Vec) instance.
   /// If the [`Vec`](alloc::vec::Vec) has a length greater than the declared capacity of the
@@ -1741,7 +1806,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// ```
   /// # use staticvec::*;
   /// let mut sv = staticvec![1, 2, 3];
-  /// let v = StaticVec::into_vec(sv);
+  /// let v = sv.into_vec();
   /// assert_eq!(v, [1, 2, 3]);
   /// ```
   #[cfg(feature = "std")]
@@ -2157,7 +2222,6 @@ impl<T, const N: usize> StaticVec<T, N> {
     res
   }
 
-  /*
   /// Returns a new StaticVec representing the symmetric difference of `self` and `other` (that is,
   /// all items present in at least one of `self` or `other`, but *not* present in both.)
   ///
@@ -2211,7 +2275,6 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
     res
   }
-  */
 
   /// Returns a new StaticVec representing the intersection of `self` and `other` (that is,
   /// all items present in both `self` and `other`.)
@@ -2250,7 +2313,6 @@ impl<T, const N: usize> StaticVec<T, N> {
     res
   }
 
-  /*
   /// Returns a new StaticVec representing the union of `self` and `other` (that is, the full
   /// contents of both `self` and `other`, minus any duplicates.)
   ///
@@ -2283,7 +2345,6 @@ impl<T, const N: usize> StaticVec<T, N> {
       res
     }
   }
-  */
 
   /// A concept borrowed from the widely-used `SmallVec` crate, this function
   /// returns a tuple consisting of a constant pointer to the first element of the StaticVec,
@@ -2553,10 +2614,6 @@ impl<const N: usize> StaticVec<u8, N> {
     // This works at compile time too, of course, thanks to the `const_panic` feature.
     assert!(
       values.len() <= N,
-      // At the moment, I don't think this message is actually printed in any context when the
-      // assertion gets triggered (currently it's just "could not evaluate static initializer") but
-      // I feel like it doesn't hurt to have here just in case the compiler-error behavior changes
-      // such that custom messages are actually shown.
       "Attempted to create a `StaticString` with insufficient capacity from an `&str` literal!"
     );
     Self::new_from_str_data(Self::bytes_to_data(values.as_bytes()), values.len())
