@@ -131,8 +131,7 @@ macro_rules! __staticsort {
           mut values: [$type; $len],
           mut low: isize,
           mut high: isize,
-        ) -> [$type; $len]
-        {
+        ) -> [$type; $len] {
           if high - low <= 0 {
             return values;
           }
@@ -190,7 +189,7 @@ macro_rules! __staticsort {
 ///
 /// # Example usage:
 /// ```
-/// #![feature(const_fn)]
+/// #![feature(const_fn, const_fn_floating_point_arithmetic)]
 ///
 /// use staticvec::{sortedstaticvec, StaticVec};
 ///
