@@ -1,6 +1,6 @@
 use core::cmp::{Ordering, PartialOrd};
 use core::intrinsics::{assume, ptr_offset_from};
-use core::mem::{size_of, MaybeUninit};
+use core::mem::{/* align_of, */ size_of, MaybeUninit};
 
 use crate::StaticVec;
 
@@ -77,8 +77,7 @@ where T: Copy {
 pub(crate) fn partial_compare<T1, T2: PartialOrd<T1>>(
   this: &[T2],
   other: &[T1],
-) -> Option<Ordering>
-{
+) -> Option<Ordering> {
   let min_length = this.len().min(other.len());
   unsafe {
     let left = this.get_unchecked(0..min_length);
@@ -100,8 +99,7 @@ pub(crate) fn quicksort_internal<T: Copy + PartialOrd>(
   values: *mut T,
   mut low: isize,
   mut high: isize,
-)
-{
+) {
   // We call this function from exactly one place where `low` and `high` are known to be within an
   // appropriate range before getting passed into it, so there's no need to check them again here.
   // We also know that `values` will never be null, so we can safely give an optimizer hint here.
@@ -149,22 +147,51 @@ pub(crate) fn quicksort_internal<T: Copy + PartialOrd>(
   }
 }
 
-/// A local `const fn` version of `slice::from_raw_parts`.
+/*
+/// A local (identically written) `const fn` version of `intrinsics::is_aligned_and_not_null`.
+#[inline(always)]
+pub(crate) const fn is_aligned_and_not_null<T>(ptr: *const T) -> bool {
+  // This does not compile currently in certain const contexts because of the `ptr as usize` cast,
+  // even though the results of the below slice functions are always valid and properly usable
+  // from regular code even when declared as `static`.
+  unsafe { !ptr.is_null() && ptr as usize % align_of::<T>() == 0 }
+}
+*/
+
+/// A local (identically written) `const fn` version of `slice::from_raw_parts`.
 #[inline(always)]
 pub(crate) const fn slice_from_raw_parts<'a, T>(data: *const T, length: usize) -> &'a [T] {
+  debug_assert!(
+    /*
+    is_aligned_and_not_null(data),
+    "Attempted to create an unaligned or null slice!"
+    */
+    // See comment starting at line 154 for more info about what's going on here.
+    !data.is_null(),
+    "Attempted to create a null slice!"
+  );
   debug_assert!(
     size_of::<T>().saturating_mul(length) <= isize::MAX as usize,
-    "Attempted to create a slice covering at least half the address space!"
+    "Attempted to create a slice covering at least half of the address space!"
   );
   unsafe { &*core::ptr::slice_from_raw_parts(data, length) }
 }
 
-/// A local `const fn` version of `slice::from_raw_parts_mut`.
+/// A local (identically written) `const fn` version of `slice::from_raw_parts_mut`.
 #[inline(always)]
 pub(crate) const fn slice_from_raw_parts_mut<'a, T>(data: *mut T, length: usize) -> &'a mut [T] {
+  debug_assert!(
+    /*
+    is_aligned_and_not_null(data),
+    "Attempted to create an unaligned or null slice!"
+    */
+    // See comment starting at line 154 for more info about what's going on here.
+    !data.is_null(),
+    "Attempted to create a null slice!"
+  );
   debug_assert!(
     size_of::<T>().saturating_mul(length) <= isize::MAX as usize,
-    "Attempted to create a slice covering at least half the address space!"
+    "Attempted to create a slice covering at least half of the address space!"
   );
   unsafe { &mut *core::ptr::slice_from_raw_parts_mut(data, length) }
 }
